/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

#include "config.h"
#include "asterisk.h"

#if AST_VER >= 120000 && AST_VER < 140000
ASTERISK_FILE_VERSION(__FILE__, "$Revision: 400356 $")
#elif AST_VER >= 140000 && AST_VER < 150000
ASTERISK_REGISTER_FILE()
#endif

#include "asterisk/linkedlists.h"
#include "asterisk/channel.h"
#include "asterisk/format.h"
#include "asterisk/utils.h"
#include "asterisk/frame.h"
#include "asterisk/bridge.h"
#if AST_VER >= 130000
#include "asterisk/format_cache.h"
#endif
#include "asterisk/app.h"
#include "asterisk/pbx.h"

#include "process_frame.h"
#include "resampled.h"


static inline void mix(short *dst, short *src, int count)
{
    int i;
    for (i = 0; i < count; i++)
        ast_slinear_saturated_add(dst++, src++);
}

static inline int need_repeat(unsigned int cnt, unsigned int repeat_cnt)
{
    return (repeat_cnt == 0 || cnt < repeat_cnt);
}

static void mix1c16b_s(unsigned int samples_offset, short *dst_data,
    size_t dst_size, short *src_data, size_t src_size, unsigned int repeat)
{
    unsigned int cnt = samples_offset / src_size;
    unsigned int data_offset = samples_offset % src_size;
    unsigned int avail;
    unsigned int need;
    unsigned int processing;
    unsigned int total = 0;

    if (!need_repeat(cnt, repeat))
        return;
    
    while (total != dst_size)
    {
        avail       = src_size - data_offset;
        need        = dst_size - total;
        processing  = (need < avail) ? need : avail;
        
        mix(dst_data + total, src_data + data_offset, processing);
        
        total += processing;
        data_offset = (data_offset + processing) % src_size;
        if (data_offset + processing == src_size)
        {
            cnt++;
            if (!need_repeat(cnt, repeat))
                return;
        }
    }    
}

struct ast_frame *process_frame(struct ast_bridge_channel *bridge_channel,
    struct process_frame_args *args, struct ast_frame *frame, int update_offset)
{
    struct process_frame_info *info;
    struct ast_format *format;
    struct translator_info *trans;
    
    if (args->cfg.enabled == 0 || frame == NULL ||
        frame->frametype != AST_FRAME_VOICE || frame->samples == 0)
    {
        return frame;
    }
        
    info = bridge_channel->tech_pvt;
    if (info == NULL)
        return frame;

#if AST_VER >= 120000 && AST_VER < 130000
    format = &frame->subclass.format;
#else
    format = frame->subclass.format;
#endif
    
    AST_LIST_LOCK(&info->translators);
    
        // get translator info
        trans = translator_info_list_get(&info->translators, format);
        if (trans == NULL)
        {
            trans = translator_info_new(args->warns, format);
            if (trans == NULL)
                goto cleanup;
            
            translator_info_list_add(&info->translators, trans);            
        }
        //ast_verbose("--> translator info get success\n");
        
        struct resampled_data *rs = trans->rs;
        
        // update offset
        if (info->rs_last != rs)
        {
            process_frame_update_rate(rs, info->rs_last, &info->samples_offset);
            info->rs_last = rs;
        }

        if (rs == NULL)
            goto cleanup;

        if (!need_repeat(info->samples_offset / rs->samples_c, args->cfg.repeat))
            goto cleanup;

        // decode frame
        if (trans->to_slinear != NULL)// to_slinear and from_slinear is not NULL
        {
            struct ast_frame *fr_decoded;
            struct ast_frame *fr_encoded;

            fr_decoded = ast_translate(trans->to_slinear, frame, 0);
            if (fr_decoded == NULL)
                goto cleanup;
            //ast_verbose("--> decode success\n");
            
            process_frame_slin(args, info, rs, fr_decoded, update_offset);
                
            fr_encoded = ast_translate(trans->from_slinear, fr_decoded, 0);
            ast_frfree(fr_decoded);
            
            if (fr_encoded == NULL)
                goto cleanup;
            //ast_verbose("--> encode success\n");
            
            ast_frfree(frame);
            frame = fr_encoded;
        }
        else
            process_frame_slin(args, info, rs, frame, update_offset);
    
cleanup:
    AST_LIST_UNLOCK(&info->translators);
    
    return frame;
}

void process_frame_slin(struct process_frame_args *args,
    struct process_frame_info *info, struct resampled_data *rs,
    struct ast_frame *frame, int update_offset)
{
    if (args->cfg.enabled == 0 || frame == NULL ||
        frame->frametype != AST_FRAME_VOICE || frame->samples == 0 ||
        rs == NULL)
    {
        return;
    }

    mix1c16b_s(info->samples_offset, frame->data.ptr, frame->samples, rs->samples, rs->samples_c, args->cfg.repeat);

    if (update_offset)
        process_frame_update_offset(args, info, rs, frame);
    
    return;
}

void process_frame_update_offset(struct process_frame_args *args,
    struct process_frame_info *info, struct resampled_data *rs,
    struct ast_frame *frame)
{
    if (args->cfg.enabled == 0 || frame == NULL ||
        frame->frametype != AST_FRAME_VOICE || frame->samples == 0 ||
        rs == NULL)
    {
        return;
    }
    
    info->samples_offset += frame->samples;
}

int process_frame_init(struct process_frame_args *args)
{
    int res;
    char *path;
    
    memset(args, 0, sizeof(struct process_frame_args));
    
    res = config_load(&args->cfg);
    if (res)
    {
        ast_log(LOG_ERROR, "Unable to load config\n");
        return 1;
    }
    
    if (args->cfg.enabled == 0)
        return 0;
    
    path = args->cfg.filename;
    
#ifdef WORKDIR
    char path_str[4096];
    if (args->cfg.filename[0] != '/')
    {
        path = path_str;
        if ((strlen(WORKDIR) + strlen(args->cfg.filename) + 2) > sizeof(path_str))
        {
            ast_log(LOG_ERROR, "Too long path\n");
            return 1;
        }

        sprintf(path, "%s/%s", WORKDIR, args->cfg.filename);
    }
#endif
    
    res = resampled_init(&args->warns, path);
    if (res)
    {
        switch (res)
        {
            case RESAMPLED_ERROR_MEMORY:
                ast_log(LOG_ERROR, "Can not allocate memory\n");
                break;
                
            case RESAMPLED_ERROR_PROCESS:
                ast_log(LOG_ERROR, "Resampling error\n");
                break;
                
            case RESAMPLED_ERROR_FILE_READ:
                ast_log(LOG_ERROR, "Unable to read file \"%s\"\n", path);
                break;
                
            case RESAMPLED_ERROR_FILE_FORMAT:
                ast_log(LOG_ERROR, "Wrong file format\n");
                break;
                
            default:
                ast_log(LOG_ERROR, "Unknown error\n");
        }
        return 1;
    }
    
    return 0;
}

void process_frame_deinit(struct process_frame_args *args)
{
    resampled_free(&args->warns);
    config_free(&args->cfg);
}

int process_frame_add_channel(struct ast_bridge_channel *bridge_channel,
    struct process_frame_args *args)
{
    if (args->cfg.enabled == 0)
        return 0;
    
    //struct ast_channel *channel = bridge_channel->chan;
    //ast_verbose("-> join channel %s\n", ast_channel_name(channel));
    
    struct process_frame_info *info = ast_calloc(1, sizeof(struct process_frame_info));
    if (info == NULL)
        return -1;

    translator_info_list_init(&info->translators);
    info->rs_last = resampled_get(args->warns, 8000);

    bridge_channel->tech_pvt = info;    
    return 0;
}

void process_frame_remove_channel(struct ast_bridge_channel *bridge_channel,
    struct process_frame_args *args)
{
    struct process_frame_info *info;
    
    if (args->cfg.enabled == 0)
        return;
    
    //struct ast_channel *channel = bridge_channel->chan;
    //ast_verbose("-> leave channel %s\n", ast_channel_name(channel));

    info = bridge_channel->tech_pvt;
    if (info == NULL)
        return;

    translator_info_list_free(&info->translators);
    ast_free(info);

    bridge_channel->tech_pvt = NULL;
}

void process_frame_update_rate(struct resampled_data *rs_new,
    struct resampled_data *rs_old, unsigned int *samples_offset)
{
    if (rs_new != rs_old && rs_new != NULL)
    {
        if (rs_old != NULL)
            *samples_offset = rs_new->samples_c * (uint64_t) *samples_offset / rs_old->samples_c;
        else
            *samples_offset = 0;
    }
}
