/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

#include "config.h"
#include "asterisk.h"

#if AST_VER >= 120000 && AST_VER < 140000
ASTERISK_FILE_VERSION(__FILE__, "$Revision: 400356 $")
#elif AST_VER >= 140000 && AST_VER < 150000
ASTERISK_REGISTER_FILE()
#endif

#include "asterisk/format.h"
#include "asterisk/translate.h"
#include "asterisk/linkedlists.h"
#if AST_VER >= 130000
#include "asterisk/format_cache.h"
#endif

#include "translate_info.h"


void translator_info_free(struct translator_info *info)
{
    if (info == NULL)
        return;
    
    if (info->to_slinear != NULL)
    {
        ast_translator_free_path(info->to_slinear);
        info->to_slinear = NULL;
    }
    
    if (info->from_slinear != NULL)
    {
        ast_translator_free_path(info->from_slinear);
        info->from_slinear = NULL;
    }
    
    ast_free(info);
}

#if AST_VER >= 120000 && AST_VER < 130000
struct translator_info *translator_info_new(resampled_array_t arr,
    struct ast_format *format)
{
    struct translator_info *info;
    unsigned int rate;
    int slin_id;
    struct ast_format slin;
    
    info = ast_calloc(1, sizeof(struct translator_info));
    if (info == NULL)
    {
        ast_log(LOG_ERROR, "Can not allocate memory\n");
        return NULL;
    }

    rate = ast_format_rate(format);
    slin_id = ast_format_slin_by_rate(rate);
    ast_format_set(&slin, slin_id, 0);

    ast_format_copy(&info->format, format);

    info->rs = resampled_get(arr, rate);
    
    if (ast_format_cmp(&info->format, &slin) == AST_FORMAT_CMP_EQUAL)
        return info;
    
    info->to_slinear = ast_translator_build_path(&slin, &info->format);
    if (info->to_slinear == NULL)
    {
        ast_log(LOG_ERROR, "Can not find translation path from %s to slin\n",
            ast_getformatname(&info->format));
        translator_info_free(info);
        return NULL;
    }
    
    info->from_slinear = ast_translator_build_path(&info->format, &slin);
    if (info->from_slinear == NULL)
    {
        ast_log(LOG_ERROR, "Can not find translation path from slin to %s\n",
            ast_getformatname(&info->format));
        translator_info_free(info);
        return NULL;
    }
    
    return info;
}
#else
struct translator_info *translator_info_new(resampled_array_t arr,
    struct ast_format *format)
{
    struct translator_info *info;
    unsigned int rate;
    struct ast_format *slin;
    
    info = ast_calloc(1, sizeof(struct translator_info));
    if (info == NULL)
    {
        ast_log(LOG_ERROR, "Can not allocate memory\n");
        return NULL;
    }

    rate = ast_format_get_sample_rate(format);
    slin = ast_format_cache_get_slin_by_rate(rate);

    info->format = ao2_bump(format);
    info->rs = resampled_get(arr, rate);
    
    if (ast_format_cmp(info->format, slin) == AST_FORMAT_CMP_EQUAL)
        return info;
    
    info->to_slinear = ast_translator_build_path(slin, info->format);
    if (info->to_slinear == NULL)
    {
        ast_log(LOG_ERROR, "Can not find translation path from %s to slin\n",
            ast_format_get_codec_name(info->format));
        translator_info_free(info);
        return NULL;
    }
    
    info->from_slinear = ast_translator_build_path(info->format, slin);
    if (info->from_slinear == NULL)
    {
        ast_log(LOG_ERROR, "Can not find translation path from slin to %s\n",
            ast_format_get_codec_name(info->format));
        translator_info_free(info);
        return NULL;
    }
    
    return info;
}
#endif

void translator_info_list_init(struct translator_info_list *translator_info_list)
{
    AST_LIST_HEAD_INIT(translator_info_list);
}

void translator_info_list_free(struct translator_info_list *translator_info_list)
{
    struct translator_info *info;
    AST_LIST_LOCK(translator_info_list);
    AST_LIST_TRAVERSE_SAFE_BEGIN(translator_info_list, info, entry)
    {
        AST_LIST_REMOVE_CURRENT(entry);        
        translator_info_free(info);
    }
    AST_LIST_TRAVERSE_SAFE_END;
    AST_LIST_UNLOCK(translator_info_list);
}

// no lock
struct translator_info *translator_info_list_get(
    struct translator_info_list *translator_info_list,
    const struct ast_format *format)
{
    struct translator_info *info;
    AST_LIST_TRAVERSE(translator_info_list, info, entry)
    {
#if AST_VER >= 120000 && AST_VER < 130000
        if (ast_format_cmp(&info->format, format) == AST_FORMAT_CMP_EQUAL)
#else
        if (ast_format_cmp(info->format, format) == AST_FORMAT_CMP_EQUAL)
#endif
            return info;
    }
    
    return NULL;
}

// no lock
void translator_info_list_add(struct translator_info_list *translator_info_list,
    struct translator_info *translator_info)
{
    AST_LIST_INSERT_TAIL(translator_info_list, translator_info, entry);
}
