/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

#include "config.h"
#include "asterisk.h"

#if AST_VER >= 120000 && AST_VER < 140000
ASTERISK_FILE_VERSION(__FILE__, "$Revision: 400356 $")
#elif AST_VER >= 140000 && AST_VER < 150000
ASTERISK_REGISTER_FILE()
#endif

#include "asterisk/config.h"
#include "asterisk/logger.h"

#include "warn_config.h"


void config_free(struct warn_config *config)
{
    if (config->filename != NULL)
    {
        ast_free(config->filename);
        config->filename = NULL;
    }
}

int config_load(struct warn_config *config)
{
    static const char config_fname[] = "warning.conf";
    struct ast_config *cfg;
    struct ast_variable *v;
    struct ast_flags config_flags = {0};
    int need_repeat = 0;

    // set defaults
    memset(config, 0, sizeof(struct warn_config));
    config->enabled = 0;
    config->repeat = 0;
    
    cfg = ast_config_load(config_fname, config_flags);
    if (cfg == NULL)
    {
        ast_log(LOG_NOTICE, "Unable to load config %s\n", config_fname);
        return 1;
    }
    
    for (v = ast_variable_browse(cfg, "general"); v; v = v->next)
    {
        if (!strcasecmp(v->name, "enabled"))
        {
            config->enabled = ast_true(v->value);
        }
        else if (!strcasecmp(v->name, "filename"))
        {
            if (config->filename != NULL)
                ast_free(config->filename);
            config->filename = ast_strdup(v->value);
        }
        else if (!strcasecmp(v->name, "repeat"))
        {
            need_repeat = ast_true(v->value);
        }
        else if (!strcasecmp(v->name, "repeat_count"))
        {
            config->repeat = atoi(v->value);
            if (config->repeat < 0)
                config->repeat = 0;
        }
    }
    
    if (!need_repeat)
        config->repeat = 1;
    
    if (config->enabled && config->filename == NULL)
    {
        ast_log(LOG_NOTICE, "File name value not exists in %s\n", config_fname);        
        return 1;
    }
    
    return 0;
}
